-- ship selection
--   StationEquipmentManageShipSelectionTab

-- load tab
--   StationCommoditiesLoadTab
--   StationCommoditiesLoadTab[2][3][2]   <- right side tree

local ship_tree_label = nil
local ship_tree = nil
local ship_tree_map = {}
local station_tree = nil
local station_tree_map = {}
local dialog_on = false

local function load_station_data()
  local item_list = {}
  local active_ship_id = GetActiveShipID()
  for _, item_info in StationSellableInventoryPairs() do
    local id = item_info.itemid
    if active_ship_id ~= id and GetInventoryItemName(GetInventoryItemContainerID(id)) == "Storage Container" then
      local item = {}
      item.id = id
      item.name = GetInventoryItemName(id)
      item.mass = tonumber(string.format("%.3f", GetInventoryItemMass(id)))
      item.type = GetInventoryItemClassType(id)
      item.info = item_info
      if item.type == CLASSTYPE_SHIP then
        local ship_inventory = GetShipInventory(id)
        item.cargo = ship_inventory.cargo
        item.addons = ship_inventory.addons
      end
      table.insert(item_list, item)
    end
  end

  table.sort(item_list,
  function(a,b)
    if a.name < b.name then return true end
    if a.name > b.name then return false end
    return tonumber(a.id) > tonumber(b.id)
  end
  )

  local unique_list = {}
  local last_item_name = ""
  for _,i in ipairs(item_list) do
    if (i.type == CLASSTYPE_SHIP or i.name ~= last_item_name) then
      table.insert(unique_list, i)
      last_item_name = i.name
    end
  end

  return unique_list
end

local function load_ship_data()
  local item_list = {}
  local active_ship_id = GetActiveShipID()

  for _,id in ipairs(GetShipInventory(active_ship_id).cargo) do
    local item = {}
    item.id = id
    item.name = GetInventoryItemName(id)
    item.type = GetInventoryItemClassType(id)
    item.quantity = GetInventoryItemQuantity(id)
    if item.type == CLASSTYPE_SHIP then
      local ship_inventory = GetShipInventory(id)
      item.cargo = ship_inventory.cargo
      item.addons = ship_inventory.addons
    end
    table.insert(item_list, item)
  end

  table.sort(item_list,
  function(a,b)
    if a.name < b.name then return true end
    if a.name > b.name then return false end
    return tonumber(a.id) > tonumber(b.id)
  end
  )

  local unique_list = {}
  local last_item_name = ""
  for _,i in ipairs(item_list) do
    if (i.type == CLASSTYPE_SHIP or i.name ~= last_item_name) then
      table.insert(unique_list, i)
      last_item_name = i.name
    end
  end

  return unique_list
end

local function update_station_tree()
  if station_tree == nil then return end

  station_tree_map = {}
  station_tree.delnode = "ALL"
  local list = load_station_data()
  local station_tree_data = {branchname="station items"}
  for _,i in ipairs(list) do
    if i.cargo ~= nil and i.addons ~= nil then
      local ship_data = {branchname="1x "..i.name}
      table.insert(station_tree_map, i.id)
      local cargo_data = {branchname="cargo"}
      table.insert(station_tree_map, i.id)
      local addons_data = {branchname="addons"}
      table.insert(station_tree_map, i.id)
      for _,c_id in pairs(i.cargo) do
        table.insert(cargo_data, GetInventoryItemQuantity(c_id).."x "..GetInventoryItemName(c_id))
        table.insert(station_tree_map, i.id)
      end
      for _,a_id in pairs(i.addons) do
        table.insert(addons_data, GetInventoryItemName(a_id))
        table.insert(station_tree_map, i.id)
      end
      table.insert(ship_data, cargo_data)
      table.insert(ship_data, addons_data)
      table.insert(station_tree_data, ship_data)
    else
      table.insert(station_tree_data, {branchname=GetInventoryItemQuantity(i.id).."x "..i.name})
      table.insert(station_tree_map, i.id)
    end

  end
  iup.TreeSetValue(station_tree, station_tree_data)

  return station_tree_map
end

local function update_ship_tree()
  if ship_tree == nil then return end

  ship_tree_map = {}
  ship_tree.delnode = "ALL"
  local list = load_ship_data()
  local ship_tree_data = {branchname="ship items"}
  for _,i in ipairs(list) do
    if i.cargo ~= nil and i.addons ~= nil then
      local ship_data = {branchname="1x "..i.name}
      table.insert(ship_tree_map, i.id)
      local cargo_data = {branchname="cargo"}
      table.insert(ship_tree_map, i.id)
      local addons_data = {branchname="addons"}
      table.insert(ship_tree_map, i.id)
      for _,c_id in pairs(i.cargo) do
        table.insert(cargo_data, GetInventoryItemQuantity(c_id).."x "..GetInventoryItemName(c_id))
        table.insert(ship_tree_map, i.id)
      end
      for _,a_id in pairs(i.addons) do
        table.insert(addons_data, GetInventoryItemName(a_id))
        table.insert(ship_tree_map, i.id)
      end
      table.insert(ship_data, cargo_data)
      table.insert(ship_data, addons_data)
      table.insert(ship_tree_data, ship_data)
    else
      table.insert(ship_tree_data, {branchname=GetInventoryItemQuantity(i.id).."x "..i.name})
      table.insert(ship_tree_map, i.id)
    end

  end
  iup.TreeSetValue(ship_tree, ship_tree_data)

  ship_tree_label.title = "Ship ("..GetActiveShipCargoCount().."/"..GetActiveShipMaxCargo()..")"

  return ship_tree_map
end

local function update_trees()
  station_tree_map = update_station_tree(station_tree)
  ship_tree_map = update_ship_tree(ship_tree)
end

local left_top_hbox = StationCommoditiesLoadTab[1][1]

local b = iup.button{title="Advanced Load"}

iup.Append(left_top_hbox, b)

function b:action()
  --print("not implemented")
  dialog_on = true

  local main_box = iup.vbox{gap=15}
  local dlg = iup.dialog{
    main_box,
    title="Load",
    expand="YES",
    active="YES",
    bgcolor="0 0 0 128 *",
    fullscreen="YES",
    topmost = "YES",
    BORDER="NO",
    RESIZE="NO",
    MAXBOX="NO",
    MINBOX="NO",
    MENUBOX="NO",
    MODAL="YES"
  }

  function dlg:close_cb()
    self:hide()
  end
  local close_btn = iup.button{title="Close", expand="YES", bgcolor="64 0 0 192 *"}
	function close_btn:action ()
    dialog_on = false
		dlg:hide()
	end

  ship_tree = iup.tree{expand="NO",size="%40x",expand="VERTICAL"}
  local ship_tree_selected_index = nil
  function ship_tree:selection_cb(id, status)
    if status ~= 1 then return end
    ship_tree_selected_index = id
  end
  station_tree = iup.tree{expand="NO",size="%40x",expand="VERTICAL"}
  local station_tree_selected_index = nil
  function station_tree:selection_cb(id, status)
    if status ~= 1 then return end
    station_tree_selected_index = id
  end

  local unload_btn = iup.button{title="Unload selected ->",size="%40x%5"}
  function unload_btn:action()
    if ship_tree_selected_index == nil or ship_tree_selected_index == 0 then return end
    local item_id = ship_tree_map[ship_tree_selected_index]
    if item_id == nil then return end
    --print(""..ship_tree_selected_index..", "..item_id)

    UnloadCargo{{itemid=item_id,quantity=GetInventoryItemQuantity(item_id)}}

    -- reset / reload data
    ship_tree_selected_index = nil
    update_trees()
  end
  local load_btn = iup.button{title="<- Load selected",size="%40x%5"}
  function load_btn:action()
    if station_tree_selected_index == nil or station_tree_selected_index == 0 then return end
    local item_id = station_tree_map[station_tree_selected_index]
    if item_id == nil then return end
    --print(""..station_tree_selected_index..", "..item_id)

    local free_space = GetActiveShipMaxCargo() - GetActiveShipCargoCount()
    local max_quantity_fits = math.floor(free_space / GetInventoryItemVolume(item_id))
    LoadCargo{{itemid=item_id,quantity=max_quantity_fits}}

    -- reset / reload data
    station_tree_selected_index = nil
    update_trees()
  end

  ship_tree_label = iup.label{title="Ship ("..GetActiveShipCargoCount().."/"..GetActiveShipMaxCargo()..")"}

  iup.Append(main_box, iup.hbox{
    iup.fill{},
    iup.vbox{
      ship_tree_label,
      iup.vbox{ship_tree},
      unload_btn,
      size="x%90"
    },
    iup.fill{size="%2x"},
    iup.vbox{
      iup.label{title="Station"},
      iup.vbox{station_tree},
      load_btn,
      size="x%90"
    },
    iup.fill{}
  })
  iup.Append(main_box, close_btn)

  update_trees()
  --plprint("map: "..dprint(station_tree_map))

  dlg:show()

end

function cargo_change(event, data)
  if not dialog_on then return end

  update_trees()
end

RegisterEvent(cargo_change, "INVENTORY_ADD")
RegisterEvent(cargo_change, "INVENTORY_RECEIVED")
RegisterEvent(cargo_change, "INVENTORY_REMOVE")
RegisterEvent(cargo_change, "INVENTORY_UPDATE")
